diff --git a/src/lib-compression/test-compression.c b/src/lib-compression/test-compression.c
index 0252d4f..62cff92 100644
--- a/src/lib-compression/test-compression.c
+++ b/src/lib-compression/test-compression.c
@@ -81,6 +81,7 @@ static void test_compression_handler(const struct compression_handler *handler)
 
 	test_assert(memcmp(input_sha1, output_sha1, sizeof(input_sha1)) == 0);
 	i_unlink(path);
+	i_close_fd(&fd);
 
 	test_end();
 }
diff --git a/src/lib-storage/list/mailbox-list-subscriptions.c b/src/lib-storage/list/mailbox-list-subscriptions.c
index 0b403fd..9a07c0f 100644
--- a/src/lib-storage/list/mailbox-list-subscriptions.c
+++ b/src/lib-storage/list/mailbox-list-subscriptions.c
@@ -143,6 +143,7 @@ int mailbox_list_subscriptions_refresh(struct mailbox_list *src_list,
 		return -1;
 	}
 	if (st.st_mtime == dest_list->subscriptions_mtime &&
+	    ST_MTIME_NSEC(st) == dest_list->subscriptions_mtime_nsec &&
 	    st.st_mtime < dest_list->subscriptions_read_time-1) {
 		/* we're up to date */
 		return 0;
@@ -152,8 +153,10 @@ int mailbox_list_subscriptions_refresh(struct mailbox_list *src_list,
 	dest_list->subscriptions_read_time = ioloop_time;
 
 	subsfile_ctx = subsfile_list_init(dest_list, path);
-	if (subsfile_list_fstat(subsfile_ctx, &st) == 0)
+	if (subsfile_list_fstat(subsfile_ctx, &st) == 0) {
 		dest_list->subscriptions_mtime = st.st_mtime;
+		dest_list->subscriptions_mtime_nsec = ST_MTIME_NSEC(st);
+	}
 	while ((name = subsfile_list_next(subsfile_ctx)) != NULL) T_BEGIN {
 		T_BEGIN {
 			ret = mailbox_list_subscription_fill_one(dest_list,
diff --git a/src/lib-storage/mailbox-list-private.h b/src/lib-storage/mailbox-list-private.h
index e0659ca..aa89ab7 100644
--- a/src/lib-storage/mailbox-list-private.h
+++ b/src/lib-storage/mailbox-list-private.h
@@ -118,6 +118,7 @@ struct mailbox_list {
 
 	struct mailbox_tree_context *subscriptions;
 	time_t subscriptions_mtime, subscriptions_read_time;
+	unsigned int subscriptions_mtime_nsec;
 
 	struct mailbox_log *changelog;
 	time_t changelog_timestamp;
